import 'dart:math';

import 'cluster_member_info.dart';

class ClusterInfo {
  var members = List<ClusterMemberInfo>.empty(growable: true);
  var inactiveMembers = List<ClusterMemberInfo>.empty(growable: true);

  late ClusterMemberInfo currentMember;

  ClusterInfo(List<String> servers, bool encryption) {
    for (int i = 0; i < servers.length; i++) {
      members.add(ClusterMemberInfo(servers[i], encryption));
    }
    currentMember = members.elementAt(0);
  }

  ClusterMemberInfo getNextMember() {
    List<ClusterMemberInfo> activeMembers = getActiveMembers();
    if (activeMembers.isEmpty) {
      inactiveMembers.clear();
      activeMembers = members;
    }
    int nextIndex = getNextIndex(activeMembers);
    currentMember = activeMembers.elementAt(nextIndex);
    return currentMember;
  }

  void setMemberInactive(ClusterMemberInfo member) {
    inactiveMembers.add(member);
  }

  ClusterMemberInfo getCurrentMember() {
    return currentMember;
  }

  List<ClusterMemberInfo> getActiveMembers() {
    var activeMembers = List<ClusterMemberInfo>.of(members);
    for (ClusterMemberInfo member in members) {
      for (ClusterMemberInfo inactiveMember in inactiveMembers) {
        if (member == inactiveMember) {
          activeMembers.remove(member);
        }
      }
    }
    return activeMembers;
  }

  int getNextIndex(List<ClusterMemberInfo> activeMembers) {
    int nextIndex = -1;

    int weightSum = 0;
    for (int i = 0; i < activeMembers.length; i++) {
      weightSum += activeMembers.elementAt(i).getWeight();
    }

    if (weightSum == 0) {
      nextIndex = (activeMembers.length * Random().nextDouble()).floor();
    } else {
      int weight = (weightSum * Random().nextDouble()).floor();
      weightSum = 0;
      for (int i = 0; i < activeMembers.length; i++) {
        weightSum += activeMembers.elementAt(i).getWeight();
        if (weightSum > weight) {
          nextIndex = i;
          break;
        }
      }
    }

    return nextIndex;
  }
}
