class ClusterMemberInfo {
  final int DEFAULT_PORT = 80;
  final int DEFAULT_PORT_ENCRYPTION = 443;
  final int DEFAULT_PRIORITY = 100;

  String unparseAddress;
  String host = "";
  int port = 0;
  int weight = 0;
  bool encryption = false;

  ClusterMemberInfo(this.unparseAddress, this.encryption) {
    weight = DEFAULT_PRIORITY;
    String address = unparseAddress;
    int q = address.indexOf(' ');
    if (q != -1) {
      weight = int.parse(address.substring(0, q));
      if (weight < 0 || weight > 100) {
        throw Exception(
            "the weight of a cluster member must be between 0 and 100, weight: $weight");
      }
      address = address.substring(q + 1);
    }

    int i = address.indexOf(']');
    int j = address.lastIndexOf(':');
    String addr;
    int port;
    if (j != -1 && j + 1 < address.length && j >= i) {
      addr = address.substring(0, j);
      port = int.parse(address.substring(j + 1));
    } else {
      addr = address;
      if (encryption) {
        port = DEFAULT_PORT_ENCRYPTION;
      } else {
        port = DEFAULT_PORT;
      }
    }

    if (port < 0 || port > 65535) {
      throw Exception("invalid port number");
    }

    if (addr == "") {
      throw Exception("cluster member with null address");
    }

    if (addr == "*") {
      throw Exception(
          "wildcard address (*) cannot be used to define a cluster member");
    }

    host = addr;
    this.port = port;
  }

  int getWeight() {
    return weight;
  }

  int getPort() {
    return port;
  }

  String getHost() {
    return host;
  }

  String getUri() {
    if (encryption) {
      return "https://$host:$port/WebSocketConnection-Secure";
    } else {
      return "http://$host:$port/WebSocketConnection";
    }
  }

  String getWsUri() {
    if (encryption) {
      return "wss://$host:$port/WebSocketConnection-Secure";
    } else {
      return "ws://$host:$port/WebSocketConnection";
    }
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is ClusterMemberInfo &&
          runtimeType == other.runtimeType &&
          host == other.host &&
          port == other.port;

  @override
  int get hashCode => host.hashCode ^ port.hashCode;

  String getUnparseAddress() {
    return unparseAddress;
  }

  @override
  String toString() {
    return "${[host = getHost(), port = getPort()]}";
  }

  getOrigin() {
    if (encryption) {
      return "https://$host:$port";
    } else {
      return "http://$host:$port";
    }
  }
}
