import 'dart:typed_data';

import '../utils/monotonic_id.dart';
import '../utils/timestamp_monotonic_id_impl.dart';
import '../migratorydata_client.dart';

class Configuration {
  int version = 6;

  String reconnectPolicy = MigratoryDataClient.TRUNCATED_EXPONENTIAL_BACKOFF;
  int reconnectTimeInterval = 20000;
  int reconnectMaxDelay = 360000;
  int quickReconnectMaxRetries = 3;
  int quickReconnectInitialDelay = 5000;

  bool encryption = false;
  int socketTimeoutSeconds = 0;
  int sslHandshakeTimeoutSeconds = 10;

  int serversDownBeforeNotify = 1;
  String entitlementToken = "";

  int pingIntervalMillis = 900000;

  final Uint8List sessionType;
  final Uint8List userAgent;

  Map<String, SubjectContext> subjects = {};
  Map<String, HttpHeaders> httpHeaders = {};

  MonotonicId monotonicId = TimestampMonotonicIdImpl();
  String externalToken = "";

  Configuration(this.sessionType, this.userAgent, this.version);

  void addSubject(List<String> sbjs, int history) {
    for (final s in sbjs) {
      subjects[s] = SubjectContext(history);
    }
  }

  void removeSubjects(List<String> sbjs) {
    for (final s in sbjs) {
      subjects.remove(s);
    }
  }

  Map<String, SubjectContext> getSubjects() {
    Map<String, SubjectContext> newMap = {};
    newMap.addAll(subjects);
    return newMap;
  }

  int getQuickReconnectMaxRetries() {
    return quickReconnectMaxRetries;
  }

  int getQuickReconnectInitialDelay() {
    return quickReconnectInitialDelay;
  }

  String getReconnectPolicy() {
    return reconnectPolicy;
  }

  int getReconnectTimeInterval() {
    return reconnectTimeInterval;
  }

  int getReconnectMaxDelay() {
    return reconnectMaxDelay;
  }

  bool isEncryption() {
    return encryption;
  }

  int getSocketTimeoutSeconds() {
    return socketTimeoutSeconds;
  }

  int getSslHandshakeTimeoutSeconds() {
    return sslHandshakeTimeoutSeconds;
  }

  int getServersDownBeforeNotify() {
    return serversDownBeforeNotify;
  }

  String getEntitlementToken() {
    return entitlementToken;
  }

  void setReconnectPolicy(String reconnectPolicy) {
    this.reconnectPolicy = reconnectPolicy;
  }

  void setReconnectTimeInterval(int reconnectTimeInterval) {
    this.reconnectTimeInterval = reconnectTimeInterval;
  }

  void setReconnectMaxDelay(int reconnectMaxDelay) {
    this.reconnectMaxDelay = reconnectMaxDelay;
  }

  void setQuickReconnectMaxRetries(int quickReconnectMaxRetries) {
    this.quickReconnectMaxRetries = quickReconnectMaxRetries;
  }

  void setQuickReconnectInitialDelay(int quickReconnectInitialDelay) {
    this.quickReconnectInitialDelay = quickReconnectInitialDelay;
  }

  void setEncryption(bool encryption) {
    this.encryption = encryption;
  }

  void setSocketTimeoutSeconds(int socketTimeoutSeconds) {
    this.socketTimeoutSeconds = socketTimeoutSeconds;
  }

  void setSslHandshakeTimeoutSeconds(int sslHandshakeTimeoutSeconds) {
    this.sslHandshakeTimeoutSeconds = sslHandshakeTimeoutSeconds;
  }

  void setServersDownBeforeNotify(int serversDownBeforeNotify) {
    this.serversDownBeforeNotify = serversDownBeforeNotify;
  }

  void setEntitlementToken(String entitlementToken) {
    this.entitlementToken = entitlementToken;
  }

  Uint8List getSessionType() {
    return sessionType;
  }

  Uint8List getUserAgent() {
    return userAgent;
  }

  void setPingIntervalMillis(int pingIntervalMillis) {
    this.pingIntervalMillis = pingIntervalMillis;
  }

  int getPingIntervalMillis() {
    return pingIntervalMillis;
  }

  int getVersion() {
    return version;
  }

  void setHttpHeader(String header, String value, bool preserveHeaderCase) {
    httpHeaders[header] = HttpHeaders(header, value, preserveHeaderCase);
  }

  Map<String, HttpHeaders> getHttpHeaders() {
    return httpHeaders;
  }

  void setExternalToken(String token) {
    externalToken = token;
  }

  String getExternalToken() {
    return externalToken;
  }

  MonotonicId getMonotonicId() {
    return monotonicId;
  }
}

class SubjectContext {
  int history;

  SubjectContext(this.history);
}

class HttpHeaders {
  final String header;
  final String value;
  final bool preserveHeaderCase;

  HttpHeaders(this.header, this.value, this.preserveHeaderCase);
}
