import 'util.dart';

class Subject {
  static final int MAX_NR_OF_CONSECUTIVE_RECOVERY = 2;
  static final int MAX_SEQ_ID = 999999999999;

  String subject;
  int history;

  int seq = 0;
  int seqId = MAX_SEQ_ID; // higher than the max seqId (2^16 = 65536)
  bool needsRecovery = false;
  int nrOfConsecutiveRecovery = 0;
  int messagesReceivedUntilRecovery = 0;

  String cacheRecoveryStatus = Util.RECOVERY_CACHE_OK;
  SubscribeType currentSubscribeType = SubscribeType.NORMAL;

  Subject(this.subject, this.history);

  int getSeq() {
    return seq;
  }

  void setSeq(int seq) {
    this.seq = seq;
    messagesReceivedUntilRecovery++;
  }

  int getSeqId() {
    return seqId;
  }

  void setSeqId(int seqId) {
    this.seqId = seqId;
  }

  String getSubject() {
    return subject;
  }

  int getHistory() {
    return history;
  }

  void markForRecovery() {
    messagesReceivedUntilRecovery = 0;

    if (hasEpoch()) {
      nrOfConsecutiveRecovery++;
    }
  }

  void unmarkFromRecovery() {
    nrOfConsecutiveRecovery = 0;
  }

  int getMessagesReceivedUntilRecovery() {
    return messagesReceivedUntilRecovery;
  }

  void setCacheRecoveryStatus(String status) {
    cacheRecoveryStatus = status;
  }

  String getCacheRecoveryStatus() {
    return cacheRecoveryStatus;
  }

  bool hasEpoch() {
    return seqId != MAX_SEQ_ID;
  }

  SubscribeType getSubscribeType() {
    SubscribeType type = SubscribeType.NORMAL;

    if (hasEpoch()) {
      if (nrOfConsecutiveRecovery >= MAX_NR_OF_CONSECUTIVE_RECOVERY) {
        if (history > 0) {
          type = SubscribeType.HISTORY;
        }
      } else {
        type = SubscribeType.RECOVERY;
      }
    } else {
      if (history > 0) {
        type = SubscribeType.HISTORY;
      }
    }

    if (type == SubscribeType.NORMAL || type == SubscribeType.HISTORY) {
      setCacheRecoveryStatus(Util.RECOVERY_CACHE_OK);
      unmarkFromRecovery();
    }
    currentSubscribeType = type;
    return type;
  }

  SubscribeType getCurrentSubscribeType() {
    return currentSubscribeType;
  }

  void setCurrentSubscribeTypeToNormal() {
    currentSubscribeType = SubscribeType.NORMAL;
  }

  @override
  String toString() {
    return "[$subject, seq: $seq, epoch: $seqId, history: $history, needsRecovery: $needsRecovery, nrOfConsecutiveRecover: $nrOfConsecutiveRecovery, cacheRecoveryStatus: $cacheRecoveryStatus]";
  }

  void reset() {
    seq = 0;
    seqId = MAX_SEQ_ID; // 70000; // higher than the max seqId (2^16 = 65536)
    needsRecovery = false;
    nrOfConsecutiveRecovery = 0;
    messagesReceivedUntilRecovery = 0;

    cacheRecoveryStatus = Util.RECOVERY_CACHE_OK;
    currentSubscribeType = SubscribeType.NORMAL;
  }
}

enum SubscribeType { NORMAL, HISTORY, RECOVERY }
