import 'migratorydata_message.dart';

/// The implementation of this interface will handle the messages received from the server for the subscribed subjects
/// as well as various status notifications.
///
/// Use the API method \link MigratoryDataClient.setListener() \endlink to register your listener implementation.
abstract class MigratoryDataListener {
  /// This method handles the messages received from the server for the subscribed subjects.
  ///
  /// \param message   a \link MigratoryDataMessage \endlink message
  ///
  onMessage(MigratoryDataMessage message);

  /// This method handles various status notifications.
  ///
  /// The possible values of the <code>type</code> parameter are:</p>
  ///
  /// \li \link MigratoryDataClient.NOTIFY_SERVER_UP \endlink indicates that the client successfully connected to the
  /// MigratoryData server provided by the <code>info</code> parameter
  ///
  /// \li \link MigratoryDataClient.NOTIFY_SERVER_DOWN \endlink indicates that the client was unable to connect to the
  /// MigratoryData server provided by the <code>info</code> parameter
  ///
  /// \li \link MigratoryDataClient.NOTIFY_DATA_SYNC \endlink indicates that, after a failover reconnection, the client
  /// successfully synchronized the subject provided by the <code>info</code> parameter. Moreover, the client received
  /// the messages published during the failover period for this subject
  ///
  /// \li \link MigratoryDataClient.NOTIFY_DATA_RESYNC \endlink indicates that, after a failover reconnection, the
  /// client successfully synchronized the subject provided by the <code>info</code> parameter. However, the client
  /// have not received the potential messages published during the failover period for this subject, the client
  /// behaving like a fresh client which just connected to the MigratoryData server
  ///
  /// \li \link MigratoryDataClient.NOTIFY_SUBSCRIBE_ALLOW \endlink indicates that the client - identified with the
  /// token given in the argument of \link MigratoryDataClient.setEntitlementToken() \endlink - is allowed to subscribe
  /// to the subject provided by the <code>info</code> parameter
  ///
  /// \li \link MigratoryDataClient.NOTIFY_SUBSCRIBE_DENY \endlink indicates that the client - identified with the
  /// token given in the argument of \link MigratoryDataClient.setEntitlementToken() \endlink - is not allowed to
  /// subscribe to the subject provided by the <code>info</code> parameter
  ///
  /// \li \link MigratoryDataClient.NOTIFY_PUBLISH_OK \endlink indicates that the client successfully published the
  /// message having the closure provided by the <code>info</code> parameter
  ///
  /// \li \link MigratoryDataClient.NOTIFY_PUBLISH_FAILED \endlink indicates that the client was unable to publish the
  /// message with the closure provided by the <code>info</code> parameter
  ///
  /// \li \link MigratoryDataClient.NOTIFY_PUBLISH_DENIED \endlink indicates that the client was not authorized to
  /// publish the message with the closure provided by the <code>info</code> parameter
  ///
	/// \li \link MigratoryDataClient.NOTIFY_CONNECT_OK \endlink indicates that the client - identified with the
	/// token given in the argument of \link MigratoryDataClient.setEntitlementToken() \endlink - is allowed to connect
	/// for the reason provided by the <code>info</code> parameter
	///
	/// \li \link MigratoryDataClient.NOTIFY_CONNECT_DENY \endlink indicates that the client - identified with the
	/// token given in the argument of \link MigratoryDataClient.setEntitlementToken() \endlink - is denied to connect
	/// for the reason provided by the <code>info</code> parameter
  /// 
  /// \param status   the type of the status notification
  /// \param info     the info of the status notification
  onStatus(String type, String info);
}
