/// This class enumerates the MigratoryData logging levels.
///
/// The available logging levels ordered by verbosity are:
/// <ul>
///  <li>ERROR (less verbose)</li>
///  <li>WARN</li>
///  <li>INFO</li>
///  <li>DEBUG</li>
///  <li>TRACE (most verbose)</li>
/// </ul>
///
/// For production usage, we recommend the default <code>INFO</code> logging level.
enum MigratoryDataLogLevel {
  /// The <code>ERROR</code> level turns on the error logs of the API.
  ERROR,

  /// The <code>WARN</code> level turns on the warning and error logs of the API.
  WARN,

  /// The <code>INFO</code> level turns on the info, warning, and error logs of the API.
  INFO,

  /// The <code>DEBUG</code> level turns on the debug, info, warning, and error logs of the API.
  DEBUG,

  /// The <code>TRACE</code> level turns on all the logs of the API.
  TRACE
}

/// The implementation of this interface will handle the log messages produced by the library.
///
/// Use the API method \link MigratoryDataClient.setLogListener() \endlink to register your log listener implementation.
abstract class MigratoryDataLogListener {
  /// This method handles the log messages outputted by the library.
  ///
  /// @param log the log message outputted by the library
  /// @param level the verbosity level of the log message
  onLog(String log, MigratoryDataLogLevel level);
}
