import 'dart:typed_data';
import 'byte_buffer.dart';

class Buffer {
  ByteBuffer buf = ByteBuffer(0);
  int length = 0;

  Buffer(int size) {
    buf = ByteBuffer(size);
  }

  void flip() {
    buf.flip();
  }

  void putArray(Uint8List src) {
    ensureWriteSpace(src.length);
    buf.putArray(src);

    length += src.length;
  }

  void put(int b) {
    ensureWriteSpace(1);
    buf.put(b);

    length += 1;
  }

  Uint8List array() {
    return buf.array();
  }

  void ensureWriteSpace(int len) {
    int capacity = buf.capacity();
    if (buf.limit < capacity) {
      return;
    }
    int position = buf.position;
    if (len <= capacity - position) {
      return;
    }
    int newCapacity = (((position + len) * 3) / 2 + 1).toInt();
    ByteBuffer buf2 = ByteBuffer(newCapacity);
    buf.flip();
    buf2.putByteBuffer(buf);
    buf = buf2;
  }

  void putArrayWithOffset(Uint8List src, int offset, int length) {
    ensureWriteSpace(length);
    buf.putArrayWithOffset(src, offset, length);
  }

  @override
  String toString() {
    return buf.toString();
  }
}
