import 'dart:typed_data';

class ByteBuffer {
  Uint8List _buffer = Uint8List(0);
  int _capacity = 4096;
  int position = 0;
  int limit = 0;

  ByteBuffer(int capacity) {
    _capacity = capacity;
    limit = _capacity;
    _buffer = Uint8List(_capacity);
  }

  int capacity() {
    return _capacity;
  }

  void put(int b) {
    if (position < limit) {
      _buffer[position] = b;
      position++;
    } else {
      throw Exception("bufferoverflow");
    }
  }

  int get() {
    if (position < limit) {
      return _buffer[position++];
    } else {
      throw Exception("bufferoverflow");
    }
  }

  int getIndex(int i) {
    if (i >= 0 && i < limit) {
      return _buffer[i];
    } else {
      throw Exception("IndexOutOfRangeException");
    }
  }

  void getArrayWithLength(Uint8List dst, int offset, int length) {
    int end = offset + length;
    for (int i = offset; i < end; i++) {
      dst[i] = get();
    }
  }

  void getArray(Uint8List dst) {
    getArrayWithLength(dst, 0, dst.length);
  }

  void putByteAtIndex(int i, int b) {
    if (i >= 0 && i < limit) {
      _buffer[i] = b;
    } else {
      throw Exception("IndexOutOfRangeException");
    }
  }

  void clear() {
    position = 0;
    limit = _capacity;
  }

  void flip() {
    limit = position;
    position = 0;
  }

  Uint8List array() {
    return _buffer;
  }

  void putArrayWithOffset(Uint8List src, int offset, int length) {
    int end = offset + length;
    for (int i = offset; i < end; i++) {
      put(src[i]);
    }
  }

  void putArray(Uint8List b) {
    putArrayWithOffset(b, 0, b.length);
  }

  void putByteBuffer(ByteBuffer buf) {
    for (int i = 0; i < buf.limit; i++) {
      put(buf.get());
    }
  }

  @override
  String toString() {
    return "[buffer=$_buffer, capacity=$_capacity, position=$position, limit=$limit]";
  }
}
