class Restricted {
  static final int NULL = 0x00;
  static final int LF = 0x0A;
  static final int CR = 0x0D;
  static final int DOUBLE_QUOTE = 0x22;
  static final int BACKSLASH = 0x5C;
}

class PushProtocol {
  static final int UPDATE_LIMIT = 0x19;
  static final int MESSAGE_LIMIT = 0x7F;
  static final int HEADER_END = 0x1E;
  static final int ESCAPE = 0x1F;

  final operations = List<Operation>.filled(128, Operation.NULL);
  final headers = List<Header>.filled(128, Header.NULL);
  final unescapes = List<int>.filled(128, -1);
  final headerTypes = List<HeaderType>.filled(128, HeaderType.NULL);

  final OPERATIONS = List<int>.filled(128, -1);
  final HEADERS = List<int>.filled(128, -1);
  final ESCAPED = List<int>.filled(128, -1);

  static final PushProtocol instance = PushProtocol._privateConstructor();

  PushProtocol._privateConstructor() {
    OPERATIONS[Operation.SUBSCRIBE.index] = 0x01;
    OPERATIONS[Operation.UNSUBSCRIBE.index] = 0x02;
    OPERATIONS[Operation.PUBLISH.index] = 0x03;
    OPERATIONS[Operation.PING.index] = 0x04;
    OPERATIONS[Operation.IFRAME.index] = 0x05;
    OPERATIONS[Operation.DISCONNECT.index] = 0x06;
    OPERATIONS[Operation.RECOVERY_CACHE.index] = 0x08;
    OPERATIONS[Operation.ENTITLEMENT_CHECK.index] = 0x09;
    OPERATIONS[Operation.PROXY.index] = 0x0C;
    OPERATIONS[Operation.CLIENT_PUBLISH.index] = 0x10;
    OPERATIONS[Operation.CLIENT_PUBLISH_RESPONSE.index] = 0x13;
    OPERATIONS[Operation.CONNECT.index] = 0x1A;
    OPERATIONS[Operation.STATUS_EVENT.index] = 0x07;
    OPERATIONS[Operation.AUTHORIZATION_TOKEN_UPDATE.index] = 0x0B;

    HEADERS[Header.SUBJECT.index] = 0x01;
    HEADERS[Header.DATA.index] = 0x02;
    HEADERS[Header.SEQ.index] = 0x03;
    HEADERS[Header.SEQ_ID.index] = 0x04;
    HEADERS[Header.ENCODING.index] = 0x05;
    HEADERS[Header.SESSION_ID.index] = 0x06;
    HEADERS[Header.DOMAIN.index] = 0x07;
    HEADERS[Header.CALLBACK.index] = 0x08;
    HEADERS[Header.IFRAME_FUNCTION.index] = 0x09;
    HEADERS[Header.ERROR.index] = 0x0B;
    HEADERS[Header.TIMESTAMP.index] = 0x0E;
    HEADERS[Header.DOUBLE_PING.index] = 0x0F;
    HEADERS[Header.CLOSURE.index] = 0x10;
    HEADERS[Header.NODE_TYPE.index] = 0x11;
    HEADERS[Header.ENTITLEMENT_TOKEN.index] = 0x13;
    HEADERS[Header.ENTITLEMENT_STATUS.index] = 0x14;
    HEADERS[Header.WORKGROUP.index] = 0x15;
    HEADERS[Header.PUBLICATION_RETAINED.index] = 0x17;
    HEADERS[Header.PUBLICATION_QOS.index] = 0x18;
    HEADERS[Header.PUSH_NOTIFICATION.index] = 0x20;
    HEADERS[Header.MESSAGE_TYPE.index] = 0x27;
    HEADERS[Header.HISTORICAL_MESSAGES.index] = 0x28;
    HEADERS[Header.USER_AGENT.index] = 0x23;
    HEADERS[Header.SESSION_TYPE.index] = 0x24;
    HEADERS[Header.SERVER_CLIENT_PING_TIME.index] = 0x25;
    HEADERS[Header.REPLY_SUBJECT.index] = 0x2C;
    HEADERS[Header.CLUSTER_TOKEN.index] = 0x30;
    HEADERS[Header.VERSION.index] = 0x2D;
    HEADERS[Header.EXTERNAL_TOKEN.index] = 0x31;
    HEADERS[Header.MONOTONIC_ID.index] = 0x2E;
    HEADERS[Header.MAX_MESSAGE_SIZE.index] = 0x1D;
    HEADERS[Header.COMPRESSION.index] = 0x26;

    ESCAPED[MESSAGE_LIMIT] = 0x01;
    ESCAPED[HEADER_END] = 0x02;
    ESCAPED[ESCAPE] = 0x03;
    ESCAPED[Restricted.NULL] = 0x04;
    ESCAPED[Restricted.LF] = 0x05;
    ESCAPED[Restricted.CR] = 0x06;
    ESCAPED[Restricted.DOUBLE_QUOTE] = 0x07;
    ESCAPED[Restricted.BACKSLASH] = 0x08;
    ESCAPED[0x21] = 0x09;
    ESCAPED[UPDATE_LIMIT] = 0x0B;

    setHeaderType(Header.SUBJECT, HeaderType.STRING);
    setHeaderType(Header.DATA, HeaderType.BYTES);
    setHeaderType(Header.SEQ, HeaderType.INT);
    setHeaderType(Header.SEQ_ID, HeaderType.INT);
    setHeaderType(Header.ENCODING, HeaderType.INT);
    setHeaderType(Header.SESSION_ID, HeaderType.INT);
    setHeaderType(Header.DOMAIN, HeaderType.BYTES);
    setHeaderType(Header.CALLBACK, HeaderType.BYTES);
    setHeaderType(Header.IFRAME_FUNCTION, HeaderType.BYTES);
    setHeaderType(Header.ERROR, HeaderType.INT);
    setHeaderType(Header.TIMESTAMP, HeaderType.UNESCAPED_BYTES);
    setHeaderType(Header.DOUBLE_PING, HeaderType.INT);
    setHeaderType(Header.ENTITLEMENT_TOKEN, HeaderType.STRING);
    setHeaderType(Header.ENTITLEMENT_STATUS, HeaderType.STRING);
    setHeaderType(Header.WORKGROUP, HeaderType.INT);
    setHeaderType(Header.PUBLICATION_RETAINED, HeaderType.INT);
    setHeaderType(Header.PUBLICATION_QOS, HeaderType.INT);
    setHeaderType(Header.PUSH_NOTIFICATION, HeaderType.STRING);
    setHeaderType(Header.MESSAGE_TYPE, HeaderType.STRING);
    setHeaderType(Header.USER_AGENT, HeaderType.STRING);
    setHeaderType(Header.SESSION_TYPE, HeaderType.INT);
    setHeaderType(Header.SERVER_CLIENT_PING_TIME, HeaderType.INT);
    setHeaderType(Header.CLOSURE, HeaderType.STRING);
    setHeaderType(Header.NODE_TYPE, HeaderType.INT);
    setHeaderType(Header.HISTORICAL_MESSAGES, HeaderType.INT);
    setHeaderType(Header.REPLY_SUBJECT, HeaderType.STRING);
    setHeaderType(Header.CLUSTER_TOKEN, HeaderType.STRING);
    setHeaderType(Header.VERSION, HeaderType.INT);
    setHeaderType(Header.EXTERNAL_TOKEN, HeaderType.STRING);
    setHeaderType(Header.MONOTONIC_ID, HeaderType.INT);
    setHeaderType(Header.MAX_MESSAGE_SIZE, HeaderType.INT);
    setHeaderType(Header.COMPRESSION, HeaderType.INT);

    for (int i = 0; i < Operation.values.length - 1; i++) {
      operations[getOperationByte(Operation.values[i])] = Operation.values[i];
    }
    for (int i = 0; i < Header.values.length - 1; i++) {
      headers[getHeaderByte(Header.values[i])] = Header.values[i];
    }
    for (int i = 0; i < 128; i++) {
      int e = escape(i);
      if (e != -1) {
        unescapes[e] = i;
      }
    }
  }

  void setHeaderType(Header hdr, HeaderType type) {
    headerTypes[getHeaderByte(hdr)] = type;
  }

  HeaderType getHeaderType(Header hdr) {
    int headerByte = getHeaderByte(hdr);
    return headerTypes[headerByte];
  }

  int getMessageLimit() {
    return MESSAGE_LIMIT;
  }

  int getHeaderEnd() {
    return HEADER_END;
  }

  int getEscape() {
    return ESCAPE;
  }

  int getOperationByte(Operation oper) {
    return OPERATIONS[oper.index];
  }

  int getHeaderByte(Header hdr) {
    return HEADERS[hdr.index];
  }

  int escape(int b) {
    return b >= 0 ? ESCAPED[b] : -1;
  }

  int getUpdateLimit() {
    return UPDATE_LIMIT;
  }

  int unescape(int b) {
    return b >= 0 ? unescapes[b] : -1;
  }

  Operation getOperation(int b) {
    if (b >= 0) {
      return operations.elementAt(b);
    } else {
      return Operation.NULL;
    }
  }

  Header getHeader(int b) {
    if (b >= 0) {
      return headers.elementAt(b);
    } else {
      return Header.NULL;
    }
  }
}

enum HeaderType { UNESCAPED_BYTES, BYTES, STRING, INT, NULL }

enum Header {
  SUBJECT,
  DATA,
  SEQ,
  SEQ_ID,
  ENCODING,
  SESSION_ID,
  DOMAIN,
  CALLBACK,
  IFRAME_FUNCTION,
  ERROR,
  TIMESTAMP,
  DOUBLE_PING,
  ENTITLEMENT_TOKEN,
  ENTITLEMENT_STATUS,
  WORKGROUP,
  PUBLICATION_RETAINED,
  PUBLICATION_QOS,
  PUSH_NOTIFICATION,
  MESSAGE_TYPE,
  USER_AGENT,
  SESSION_TYPE,
  SERVER_CLIENT_PING_TIME,
  CLOSURE,
  NODE_TYPE,
  HISTORICAL_MESSAGES,
  REPLY_SUBJECT,
  CLUSTER_TOKEN,
  VERSION,
  EXTERNAL_TOKEN,
  MONOTONIC_ID,
  MAX_MESSAGE_SIZE,
  COMPRESSION,

  NULL
}

enum Operation {
  SUBSCRIBE,
  UNSUBSCRIBE,
  PUBLISH,
  PING,
  IFRAME,
  DISCONNECT,
  RECOVERY_CACHE,
  ENTITLEMENT_CHECK,
  PROXY,
  CLIENT_PUBLISH,
  CLIENT_PUBLISH_RESPONSE,
  CONNECT,
	STATUS_EVENT,
  AUTHORIZATION_TOKEN_UPDATE,

  NULL
}

class HttpEncoding {
  final int code;

  static final HttpEncoding WEB_SOCKET = HttpEncoding._privateConstructor(8);

  HttpEncoding._privateConstructor(this.code);
}

class SessionType {

  final int code;

  static final SessionType WEB = SessionType._privateConstructor(1);
  static final SessionType MOBILE = SessionType._privateConstructor(2);
  static final SessionType DESKTOP = SessionType._privateConstructor(3);
  static final SessionType REST = SessionType._privateConstructor(4);

  SessionType._privateConstructor(this.code);
}

class MessageType {
  final String value;

  static final MessageType SNAPSHOT = MessageType._privateConstructor("1");
  static final MessageType UPDATE = MessageType._privateConstructor("2");
  static final MessageType RECOVERED = MessageType._privateConstructor("3");
  static final MessageType HISTORICAL = MessageType._privateConstructor("4");

  MessageType._privateConstructor(this.value);
}

class EntitlementStatus {
  final String value;

  static final EntitlementStatus DENY = EntitlementStatus._privateConstructor("d");
  static final EntitlementStatus ALLOW = EntitlementStatus._privateConstructor("a");

  EntitlementStatus._privateConstructor(this.value);

  const EntitlementStatus({required this.value});
}

class PushError {
  final int value;

  static final PushError UNKNOWN_SESSION_ID = PushError._privateConstructor(0);
  static final PushError INVALID_AGENT_TYPE = PushError._privateConstructor(1);
  static final PushError TIMEOUT = PushError._privateConstructor(2);
  static final PushError CONNECT_DENY = PushError._privateConstructor(3);

  PushError._privateConstructor(this.value);

  const PushError({required this.value});
}