import 'dart:typed_data';

import 'push_protocol.dart';

class PushProtocolTransportCodec {
  static final PushProtocol pushProtocol = PushProtocol.instance;

  static final PushProtocolTransportCodec instance =
      PushProtocolTransportCodec._privateConstructor();

  PushProtocolTransportCodec._privateConstructor();

  Uint8List escape(Uint8List bytes) {
    int escapes = 0;
    for (int i = 0; i < bytes.length; i++) {
      if (pushProtocol.escape(bytes[i]) != -1) {
        escapes++;
      }
    }
    if (escapes == 0) {
      return bytes;
    }
    Uint8List ret = Uint8List(bytes.length + escapes);
    for (int i = 0, j = 0; i < bytes.length; i++, j++) {
      int escaped = pushProtocol.escape(bytes[i]);
      if (escaped != -1) {
        ret[j] = pushProtocol.getEscape();
        ret[j + 1] = escaped;
        j++;
      } else {
        ret[j] = bytes[i];
      }
    }
    return ret;
  }

  Uint8List unescape(
      Uint8List buf, int pos, int len, PushProtocol pushProtocol) {
    int escapes = 0;
    for (int i = pos; i < pos + len; i++) {
      if (buf.elementAt(i) == pushProtocol.getEscape()) {
        escapes++;
      }
    }
    Uint8List ret = Uint8List(len - escapes);
    for (int i = pos, j = 0; i < pos + len; i++, j++) {
      int b = buf.elementAt(i);
      if (b == pushProtocol.getEscape()) {
        if (i + 1 < pos + len) {
          ret[j] = pushProtocol.unescape(buf.elementAt(i + 1));
          if (ret[j] == -1) {
            throw Exception("unescape1");
          }
          i++;
        } else {
          throw Exception("unescape2");
        }
      } else {
        ret[j] = b;
      }
    }
    return ret;
  }

  Uint8List integerToBytes(int val) {
    if ((val & 0xFFFFFF80) == 0) {
      int i = pushProtocol.escape(val);
      if (i == -1) {
        return Uint8List.fromList([val]);
      } else {
        return Uint8List.fromList([pushProtocol.getEscape(), i]);
      }
    }

    int valBits;
    if ((val & 0xFF000000) != 0) {
      valBits = 24;
    } else if ((val & 0x00FF0000) != 0) {
      valBits = 16;
    } else {
      valBits = 8;
    }

    Uint8List ret = Uint8List(10);
    int retPos = 0;
    int ret7bitIndex = 0;

    while (valBits >= 0) {
      int b = ((val >> valBits) & 0xFF);
      ret7bitIndex++;
      ret[retPos] |= (b >= 0 ? b : b + 256) >> ret7bitIndex;

      int escaped = pushProtocol.escape(ret[retPos]);
      if (escaped != -1) {
        ret[retPos] = pushProtocol.getEscape();
        ret[retPos + 1] = escaped;
        retPos++;
      }

      retPos++;
      ret[retPos] |= (b << (7 - ret7bitIndex)) & 0x7F;
      valBits -= 8;
    }
    int escaped = pushProtocol.escape(ret[retPos]);
    if (escaped != -1) {
      ret[retPos] = pushProtocol.getEscape();
      ret[retPos + 1] = escaped;
      retPos++;
    }
    retPos++;

    if (retPos < ret.length) {
      Uint8List ret2 = Uint8List(retPos);
      List.copyRange(ret2, 0, ret, 0, ret2.length);
      ret = ret2;
    }

    return ret;
  }

  int bytesToInteger(
      Uint8List buf, int pos, int len, PushProtocol pushProtocol) {
    int ret = 0;
    int val7bitIndex = -1;
    int val = 0;
    int raw;
    int b;

    if (len == 1) {
      return buf.elementAt(pos);
    } else if (len == 2 && buf.elementAt(pos) == pushProtocol.getEscape()) {
      b = pushProtocol.unescape(buf.elementAt(pos + 1));
      if (b != -1) {
        return b;
      } else {
        throw Exception("bytesToInteger1");
      }
    }

    for (; len > 0; len--) {
      b = buf.elementAt(pos);
      pos++;
      if (b == pushProtocol.getEscape()) {
        if (len - 1 < 0) {
          throw Exception("bytesToInteger2");
        }
        len--;
        b = buf.elementAt(pos);
        pos++;
        raw = pushProtocol.unescape(b);
        if (raw == -1) {
          throw Exception("bytesToInteger3");
        }
      } else {
        raw = b;
      }
      if (val7bitIndex > 0) {
        val |= raw >> val7bitIndex;
        ret = ret << 8 | (val >= 0 ? val : val + 256);
        val = (raw << (8 - val7bitIndex));
      } else {
        val = (raw << -val7bitIndex);
      }
      val7bitIndex = (val7bitIndex + 7) % 8;
    }
    return ret;
  }
}
